#include <vector>
#include <stack>
#include <assert.h>     

using namespace std;

typedef unsigned int Var;
typedef unsigned int ClaInd;

/**************************************************************/
class Lit {
/**************************************************************/
public:
  int     x;
  // Use this as a constructor:
  Lit mkLit(Var var, bool sign = false) { Lit p; p.x = var + var + (int)sign; return p; }
  bool operator == (Lit p) const { return x == p.x; }
  bool operator != (Lit p) const { return x != p.x; }
};

inline  Lit  operator ~(Lit p)              { Lit q; q.x = p.x ^ 1; return q; }
inline  bool sign      (Lit p)              { return p.x & 1; }
inline  Var  var       (Lit p)              { return p.x >> 1; }
inline  void printLit  (Lit p)              { fprintf(stdout,"%d ", (sign(p)?1:-1)*(var(p)+1)); }

/**************************************************************/
class Clause {
  /**************************************************************/
  vector< Lit > clause;
 public:

  inline unsigned int size() {return clause.size();}

  inline Lit& operator[] (unsigned int i) { assert(i>=0 && i<clause.size()); return clause[i]; }

  void print()
  {
    for (unsigned int j=0; j<clause.size(); j++){
      printLit(clause[j]);
      printf(" ");
    }
    fprintf(stdout,"0\n");
  }

  inline void addlit(Lit l) {clause.push_back(l);}

  bool addedvar(Var v)
    { 
      for (unsigned int i=0; i<clause.size(); i++)
	if (var(clause[i]) == v) return true;
      return false;
    }
};
  
/**************************************************************/
class Formula {
/**************************************************************/
  Var n;
  ClaInd m;
  vector< Clause > formula;
  vector< vector < ClaInd > > occur;
 public:

 Formula() : n(0), m(0) { }

  int numvar() { return (int)n; }
  int numclauses() { return (int)m; }
  int size() { int c=0; for (ClaInd ci=0; ci<m; ci++) c += formula[ci].size(); return c; }

  void addclause(Clause c) { 
    ClaInd ci = m;
    for (unsigned int i=0; i<c.size(); i++) {
      Var v = var(c[i]);
      if (n <= v) {
	n = v + 1;
	occur.resize(v+1);
      }
      occur[v].push_back(ci);
    }
    formula.push_back(c); 
    m++;
  }

  inline Clause& operator[] (unsigned int i) { assert(i>=0 && i<formula.size()); return formula[i]; }

  inline void clear() { formula.clear(); occur.clear(); n=0; m=0; }

  void print()
  {
    printf("p cnf %d %d\n", (int)n, (int)m);
    for (ClaInd i=0; i<m; i++)
      formula[i].print();
  }

  void neato()
  {
    printf("graph {\nsize=\"10,10\";\nnode [fixedsize=true,width=0.1,height=0.08, style=filled, color=blue];\n");
    for (ClaInd i=0; i<m; i++) {
      printf("c%d  --  { ",i+1);
      for (unsigned int j=0; j<formula[i].size(); j++) {
	if ((float)var(formula[i][j]) < (float)n/10)
	  printf("%d[color=red]  ", var(formula[i][j])+1);
	else 
	  printf("%d[color=green]  ", var(formula[i][j])+1);
      }
      printf("};\n");
    }
    printf("}\n");
  }

  void principal_component()
  {
    fprintf(stderr, "Computing principal component\n\n");
    assert(m==formula.size() && n==occur.size());
    vector < unsigned int > cv(n, 0);   // component where belongs variable
    vector < unsigned int > cc(m, 0);   // component where belongs clause
    unsigned int nc = 0;       // number of components
    unsigned int maxsize = 0;  // Size of largest community
    unsigned int maxc = 0;     // Largest community
    vector < Var > tradv(n);      // Translation of variables
    vector < ClaInd > tradc(m); // Translation of clauses

    // Compute in which community is every variable and clause
    for (Var v=0; v<n; v++) {
      if (v%1000 == 0)fprintf(stderr, "\e[A%3.0f%%\n", (double)v/n*100);

      if (cv[v] == 0) {
	nc++;
	//fprintf(stderr,"\nComunity %d\n",nc);
	unsigned int size = 0;
	stack < Var > s;
	s.push(v);
	while(!s.empty()) {
	  Var v2 = s.top();
	  s.pop();
	  if (cv[v2] == 0) {
	    cv[v2] = nc;
	    //fprintf(stderr,"Variable %d\n",v2+1);
	    for (unsigned int i=0; i<occur[v2].size(); i++) {
	      ClaInd ci = occur[v2][i];
	      if (cc[ci] == 0) {
		cc[ci] = nc;
		//fprintf(stderr,"Clause %d\n",ci+1);
		size++;
		for (unsigned int j=0; j<formula[ci].size(); j++)
		  s.push(var(formula[ci][j]));
	      }
	    }
	  }
	}
	if (maxsize < size) {
	  maxsize = size;
	  maxc = nc;
	}
      }
    }
    fprintf(stderr, "Found %d comunities, largest sized %2.2f%%\n", nc, (double)maxsize/m*100);
    //fprintf(stderr, "123 %f %f\n", (float)m/n, (double)maxsize/m);

    // Compute traduction for variables and clauses
    Var v=0;
    for (Var i=0; i<tradv.size(); i++) {
      if (cv[i] == maxc) tradv[i] = v++;
      else tradv[i] = 0;
    }
    ClaInd ci=0;
    for (ClaInd i=0; i<tradc.size(); i++) {
      if (cc[i] == maxc) tradc[i] = ci++;
      else tradc[i] = 0;
    }

    // Remove all clauses and variables exept those belonging to largest community
    ci=0;
    for (ClaInd i=0; i<m; i++) {
      if (cc[i] == maxc) {
	for (unsigned int k=0; k<formula[i].size(); k++) {
	  Lit l = formula[i][k];
	  formula[i][k] = l.mkLit(tradv[var(l)], sign(l));
	}
	formula[ci++] = formula[i];
      }
    }
    formula.resize(ci);
    m=ci;
    v=0;
    for (Var i=0; i<n; i++) {
      if (cv[i] == maxc) {
	for (unsigned int k=0; k<occur[i].size(); k++) {
	  occur[i][k] = tradc[occur[i][k]];
	}
	occur[v++] = occur[i];
      }
    }
    occur.resize(v);
    n=v;
    fprintf(stderr, "New formula size n=%d m=%d k=%f\n", (int)n, (int)m, this->size()/(double)m);
    //fprintf(stderr, "123 %f %f\n", (float)m/n, (double)maxsize/m);

  }

};

