#include <vector>
#include <assert.h>     

using namespace std;

/**************************************************************/
double frandom(void)
/**************************************************************/
/* generates a random number on [0,1) with 64-bit resolution */
{
  double a = 0;
  int i;

  for (i=0; i<8; i++)
    a = (a + rand() % 256) / 256;
  return a;
}

/**************************************************************/
class Probability {
/**************************************************************/
  vector <double> p; // Probability distribution
  vector <double> a; // Cumulative probability distribution
  bool compa;        // Indicates if comulative probability has been already computed
  bool normalized;
  bool isuniform;    // Indicates that the prob. distribution is uniform
  unsigned int N;

 public:

 Probability() : compa(false), normalized(false), isuniform(false), N(0) { }

  double& operator[] (unsigned int i) { 
    if (isuniform) { 
      isuniform = false;
      assert(N>0); 
      p.resize(N);
      for (unsigned int i=0; i<N; i++) 
	p[i] = 1.0 / N;
   }
    assert(i>=0 && i<p.size()); 
    normalized = false; 
    return p[i]; 
  }

/**************************************************************/
  void print()
/**************************************************************/
  {
    for (unsigned int i=0; i<p.size(); i++)
      printf("p[%d]=%f\n", i, p[i]);
  }

/**************************************************************/
  int powerlawvar(double alpha, int minimum)
/**************************************************************/
  {
    static bool computed_den = false;
    static double alphaold;
    static double den;
    
    if (!computed_den || alpha != alphaold) {
      double denold;
      computed_den = true;
      alphaold = alpha;
      int i = 1;
      den = 0;
      denold = -1;
      while ((den > denold) && (i<1000)) {
	denold = den;
	den = den + pow((double)(i>minimum ? i : minimum), -alpha);
	i++;
      }
    }
    
    double r = frandom();
    
    int i = 1;
    double p;
    p = pow((double)(i>minimum ? i : minimum), -alpha) / den;
    while ((r > p) && (i<1000)) {
      i++;
      r = r - p;
      p = pow((double)(i>minimum ? i : minimum), -alpha) / den;
    }
    return i;
  }

/**************************************************************/
int choose()
/**************************************************************/
/* Choose a value i \in [0..n-1] with probability p[i] */
{
  if (isuniform) return(rand() % N);

  acumulate();

  int n = p.size();
  int i=0, j=n-1;
  double c = frandom();

  while (i != j) {
    int m = (j+i)/2;
    if(a[m] <= c) i = m + 1;
    else {
      if (m!=0 && a[m-1] > c) j = m -1;
      else return m;
    }
  }
  return i;
}

/**************************************************************/
void uniform(int n)
/**************************************************************/
/* Computes a uniform probability */
{
  isuniform = true;
  N = n;
}

/**************************************************************/
void geometric(int n, double b)
/**************************************************************/
/* Compute p[0] as the provability of the most frequent variable,
     then p[i] = p[i-1] + p[0] / bn^i    */
{
  double bn;
  int i;
  p.resize(n);

  if (b > 1) {
    bn =  pow(b, 1/(double)n);
    p[0] = (1 - 1/bn) / (1 - 1/b);
  }
  else {
    bn = 1;
    p[0] = 1 / (double) n;
  }
  
  for(i=1; i<n; i++) 
    p[i] = p[i-1] / bn;
  normalized = true;
}

/**************************************************************/
void powerlaw(int n, double b, double min)
/**************************************************************/
/* Compute p[0] as the provability of the most frequent variable,
     then p[i] = p[i-1] + pow( i + 0.01 * n, -b)  */
{
  double addition = 0;
  int i;
  p.resize(n);
  
  for(i=0; i<n; i++) {
    //p[i] = pow(i + 0.5 + 0.01 * (double)n, -b);
    p[i] = pow((double)i+1, -b);
    addition = addition + p[i];
  }

    for(i=0;  i<n;  i++) // Normalize
  //    p[i] = min / n + (1 - min)* p[i] / addition;
      p[i] = p[i] / addition;
  normalized = true;
}

/**************************************************************/
void acumulate()
/**************************************************************/
/* computes the acumulated provability, if necessary*/
{
  if (!normalized) {
    //fprintf(stderr, "Normalizing probability distribution\n");
    double norm = 0;
    for (unsigned int i=0; i<p.size(); i++) norm += p[i];
    assert(norm>0);
    for (unsigned int i=0; i<p.size(); i++) p[i] /= norm;
    normalized = true;
    compa = false;
  }
      
  if (!compa) {
    //fprintf(stderr, "Computing cummulative distribution\n");
    compa = true;
    unsigned int n = p.size();
    a.resize(n);
    a[0] = p[0];
    for (unsigned int i=1; i<n; i++)
      a[i] = a[i-1] + p[i];
    a[n-1] = 1;                  // This must be the value, up to rounding errors
  }
  
}
/**************************************************************/
vector <int> floor(int k)
/**************************************************************/
/* Given a provability p[i], and $k$ balls, 
   computes v[i] such that floor(p[i]*k) < v[i] < floor(p[i]*k)+1 
   and sum_{i=0}^n v[i] = k */
{
  unsigned int n = p.size();
  acumulate();

  /******* Assign pk[i] = floor(p[i]*k) *******/
  
  int tv = 0;
  vector <int> pk(n);
  
  for(unsigned int i=0; i<n; i++) {
    pk[i] = (int)(p[i] * (double)k);
    tv += pk[i];
  }
  
  // Assign the rest of balls until we get k balls
  while (tv < k) {
    double max = 0;
    int mv = 0;
    
    // look for the maximal k * p[i] - pk[i]
    for (unsigned int i=0; i< n; i++)
      if ((double)k * p[i] - pk[i] > max) {
	max = (double)k * p[i] - pk[i];
	mv = i;
      }
    pk[mv]++;
    tv++;
  }
  return pk;
}



};
  
